# -*- coding: utf-8 -*-
"""Event subscriber handler for IProfileImportedEvent."""
from collective.fingerpointing.config import AUDIT_MESSAGE
from collective.fingerpointing.interfaces import IFingerPointingSettings
from collective.fingerpointing.logger import log_info
from collective.fingerpointing.utils import get_request_information
from plone import api
from zope.component.interfaces import ComponentLookupError


def profile_imports_logger(event):
    """Log Generic Setup profile imports."""
    name = IFingerPointingSettings.__identifier__ + '.audit_profile_imports'
    try:
        audit_enabled = api.portal.get_registry_record(name, default=False)
    except ComponentLookupError:  # adding Plone site
        return

    if not audit_enabled:
        return

    try:
        user, ip = get_request_information()
    except AttributeError:
        # XXX: getRequest() returns None instead actual request
        #      on tests of this subscriber
        user, ip = '-', '-'

    info = event.tool.getProfileInfo(event.profile_id)
    action = 'profile imported'
    extras = 'id={0} version={1}'.format(
        info['id'],
        info.get('version', '-'),  # uninstall profiles have no version info
    )
    log_info(AUDIT_MESSAGE.format(user, ip, action, extras))



# -*- coding: utf-8 -*-
# BBB: remove on deprecation of Plone 4.3
from collective.fingerpointing.config import PROJECTNAME
from plone import api


def uninstall(portal, reinstall=False):
    if not reinstall:
        profile = 'profile-{0}:uninstall'.format(PROJECTNAME)
        setup_tool = api.portal.get_tool('portal_setup')
        setup_tool.runAllImportStepsFromProfile(profile)
        return 'Ran all uninstall steps.'



# -*- coding: utf-8 -*-
from collective.fingerpointing.testing import INTEGRATION_TESTING

import unittest


class UtilsTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.request = self.layer['request']

    def test_get_request_information(self):
        from collective.fingerpointing import utils
        self.assertEqual(
            utils.get_request_information(), ('test_user_1_', 'None'))

    def test_get_request_information_anonymous(self):
        from plone.app.testing import logout
        logout()
        from collective.fingerpointing import utils
        self.assertEqual(utils.get_request_information(), ('-', 'None'))

    def test_get_request_information_cloudflare(self):
        self.request.environ['HTTP_CF_CONNECTING_IP'] = '192.168.1.1'
        from collective.fingerpointing import utils
        self.assertEqual(
            utils.get_request_information(), ('test_user_1_', '192.168.1.1'))

    def test_audit_log_files(self):
        from collective.fingerpointing import utils
        audit_log_files = utils.audit_log_files()
        # we have at least one log file to process
        self.assertGreaterEqual(len(audit_log_files), 1)
        # the first file is the current audit log
        self.assertEqual('/audit.log', audit_log_files[0][-10:])


class UtilsOneLargeLogFileTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        # fresh audit log
        from collective.fingerpointing.config import fingerpointing_config
        from collective.fingerpointing.logger import log_info
        from collective.fingerpointing.utils import audit_log_files
        import os
        for filename in audit_log_files():
            os.remove(filename)
        log_info.configure(fingerpointing_config)

    def test_audit_log_lines_of(self):
        from collective.fingerpointing.logger import log_info
        # prepare some loglines
        log_info('1')
        log_info('2')
        log_info('3')
        from collective.fingerpointing import utils
        audit_log_files = utils.audit_log_files()
        self.assertEqual(len(audit_log_files), 1)
        lines_gen = utils.audit_log_lines_of(audit_log_files[0])
        self.assertEqual(next(lines_gen)[-1], '3')
        self.assertEqual(next(lines_gen)[-1], '2')
        self.assertEqual(next(lines_gen)[-1], '1')


class UtilsMultipleLogFilesTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        # create more than one logfile
        from collective.fingerpointing.config import fingerpointing_config
        from collective.fingerpointing.logger import log_info
        from collective.fingerpointing.utils import audit_log_files
        import os
        for filename in audit_log_files():
            os.remove(filename)
        fingerpointing_config['audit-log-max-size'] = 80
        fingerpointing_config['audit-log-old-files'] = 10
        log_info.configure(fingerpointing_config)

    def tearDown(self):
        from collective.fingerpointing.config import fingerpointing_config
        from collective.fingerpointing.logger import log_info
        del fingerpointing_config['audit-log-max-size']
        log_info.configure(fingerpointing_config)

    def test_all_audit_log_lines(self):
        from collective.fingerpointing.logger import log_info
        # prepare some loglines to create 2 files
        log_info('1')
        log_info('2')
        log_info('3')
        log_info('4')
        # check for min 2 files
        from collective.fingerpointing import utils
        audit_log_files = utils.audit_log_files()
        self.assertGreaterEqual(len(audit_log_files), 2)
        # check if we get all entries reversed
        lines_gen = utils.all_audit_log_lines()
        self.assertEqual(next(lines_gen)[-1], '4')
        self.assertEqual(next(lines_gen)[-1], '3')
        self.assertEqual(next(lines_gen)[-1], '2')
        self.assertEqual(next(lines_gen)[-1], '1')



# -*- coding: utf-8 -*-
"""Tests for lifecycle subscriber.

Events are slightly different among Archetypes and Dexterity.
Also, Dexterity content types have different names.
"""
from collective.fingerpointing.testing import INTEGRATION_TESTING
from collective.fingerpointing.testing import IS_PLONE_5
from collective.fingerpointing.testing import QIBBB
from logging import INFO
from plone import api
from testfixtures import LogCapture

import unittest


class LifeCycleSubscribersTestCase(unittest.TestCase, QIBBB):
    """Tests content type life cycle subscribers."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        with api.env.adopt_roles(['Manager']):
            self.folder = api.content.create(self.portal, 'Folder', 'folder')

    def test_file_lifecicle(self):
        if IS_PLONE_5:
            expected = (
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=create object=<File at /plone/folder/foo>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=modify object=<Folder at /plone/folder>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=remove object=<File at /plone/folder/foo>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=modify object=<Folder at /plone/folder>'),  # noqa: E501
            )
        else:
            expected = (
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=create object=<ATFile at /plone/folder/foo>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=modify object=<ATFolder at /plone/folder>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=modify object=<ATFile at /plone/folder/foo>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=remove object=<ATFile at /plone/folder/foo>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=modify object=<ATFolder at /plone/folder>'),  # noqa: E501
            )

        with LogCapture('collective.fingerpointing', level=INFO) as log:
            obj = api.content.create(self.folder, 'File', 'foo')
            obj.reindexObject()
            api.content.delete(obj)
            log.check(*expected)

    def test_folder_lifecicle(self):
        if IS_PLONE_5:
            expected = (
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=create object=<Folder at /plone/folder/foo>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=modify object=<Folder at /plone/folder>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=remove object=<Folder at /plone/folder/foo>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=modify object=<Folder at /plone/folder>'),  # noqa: E501
            )
        else:
            expected = (
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=create object=<ATFolder at /plone/folder/foo>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=modify object=<ATFolder at /plone/folder>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=modify object=<ATFolder at /plone/folder/foo>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=remove object=<ATFolder at /plone/folder/foo>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=modify object=<ATFolder at /plone/folder>'),  # noqa: E501
            )

        with LogCapture('collective.fingerpointing', level=INFO) as log:
            obj = api.content.create(self.folder, 'Folder', 'foo')
            obj.reindexObject()
            api.content.delete(obj)
            log.check(*expected)

    def test_image_lifecicle(self):
        if IS_PLONE_5:
            expected = (
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=create object=<Image at /plone/folder/foo>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=modify object=<Folder at /plone/folder>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=remove object=<Image at /plone/folder/foo>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=modify object=<Folder at /plone/folder>'),  # noqa: E501
            )
        else:
            expected = (
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=create object=<ATImage at /plone/folder/foo>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=modify object=<ATFolder at /plone/folder>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=modify object=<ATImage at /plone/folder/foo>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=remove object=<ATImage at /plone/folder/foo>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=modify object=<ATFolder at /plone/folder>'),  # noqa: E501
            )

        with LogCapture('collective.fingerpointing', level=INFO) as log:
            obj = api.content.create(self.folder, 'Image', 'foo')
            obj.reindexObject()
            api.content.delete(obj)
            log.check(*expected)

    def test_susbcriber_ignored_when_package_not_installed(self):
        # content type life cycle events should not raise errors
        # if package is not installed
        self.uninstall()  # BBB: QI compatibility

        obj = api.content.create(self.folder, 'Folder', 'foo')
        obj.reindexObject()
        api.content.delete(obj)



# -*- coding: utf-8 -*-
from collective.fingerpointing.testing import INTEGRATION_TESTING
from collective.fingerpointing.testing import QIBBB
from logging import INFO
from plone import api
from plone.app.discussion.interfaces import IDiscussionSettings
from plone.registry import field
from plone.registry.interfaces import IRegistry
from plone.registry.record import Record
from testfixtures import LogCapture
from zope.component import getUtility

import unittest


class RegistrySubscribersTestCase(unittest.TestCase, QIBBB):
    """Tests for plone.app.registry subscribers."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

    def test_record_modified(self):
        with LogCapture('collective.fingerpointing', level=INFO) as log:
            record = IDiscussionSettings.__identifier__ + '.globally_enabled'
            api.portal.set_registry_record(record, False)
            log.check(
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=modify object=<Record plone.app.discussion.interfaces.IDiscussionSettings.globally_enabled> value=False'),  # noqa: E501
            )

    def test_new_record_modified(self):
        registry = getUtility(IRegistry)
        registry.records['collective.fingerpointing.new_field'] = Record(
            field.TextLine(title=u'New field'))
        with LogCapture('collective.fingerpointing', level=INFO) as log:
            api.portal.set_registry_record('collective.fingerpointing.new_field', u'Descrição')  # noqa: E501
            log.check(
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=modify object=<Record collective.fingerpointing.new_field> value=Descrição'),  # noqa: E501
            )

    def test_susbcriber_ignored_when_package_not_installed(self):
        # registry events should not raise errors
        # if package is not installed
        self.uninstall()  # BBB: QI compatibility

        record = IDiscussionSettings.__identifier__ + '.globally_enabled'
        api.portal.set_registry_record(record, False)



# -*- coding: utf-8 -*-
"""Tests for workflow subscriber."""
from collective.fingerpointing.interfaces import IFingerPointingSettings
from collective.fingerpointing.testing import INTEGRATION_TESTING
from collective.fingerpointing.testing import IS_PLONE_5
from collective.fingerpointing.testing import QIBBB
from logging import INFO
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from testfixtures import LogCapture

import unittest


class WorkflowSubscribersTestCase(unittest.TestCase, QIBBB):
    """Tests content type life cycle subscribers."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        # disable lifecycle audit
        record = IFingerPointingSettings.__identifier__ + '.audit_lifecycle'
        api.portal.set_registry_record(record, False)

    def test_workflow_transitions(self):

        if IS_PLONE_5:
            expected = (
                ('collective.fingerpointing', 'INFO', u'user=test_user_1_ ip=None action=workflow transition object=<NewsItem at foo> transition=submit'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', u'user=test_user_1_ ip=None action=workflow transition object=<NewsItem at foo> transition=publish'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', u'user=test_user_1_ ip=None action=workflow transition object=<NewsItem at foo> transition=retract'),  # noqa: E501
            )
        else:
            expected = (
                ('collective.fingerpointing', 'INFO', u'user=test_user_1_ ip=None action=workflow transition object=<ATNewsItem at foo> transition=submit'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', u'user=test_user_1_ ip=None action=workflow transition object=<ATNewsItem at foo> transition=publish'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', u'user=test_user_1_ ip=None action=workflow transition object=<ATNewsItem at foo> transition=retract'),  # noqa: E501
            )

        with LogCapture('collective.fingerpointing', level=INFO) as log:
            obj = api.content.create(self.portal, 'News Item', 'foo')
            api.content.transition(obj=obj, transition='submit')
            api.content.transition(obj=obj, transition='publish')
            api.content.transition(obj=obj, transition='retract')
            log.check(*expected)

    def test_susbcriber_ignored_when_package_not_installed(self):
        # events should not raise errors if package is not installed
        self.uninstall()  # BBB: QI compatibility

        obj = api.content.create(self.portal, 'News Item', 'foo')
        api.content.transition(obj=obj, transition='submit')
        api.content.transition(obj=obj, transition='publish')
        api.content.transition(obj=obj, transition='retract')



# -*- coding: utf-8 -*-
from collective.fingerpointing.config import fingerpointing_config
from collective.fingerpointing.config import LOG_FORMAT
from collective.fingerpointing.config import PROJECTNAME

import logging
import time
import zc.lockfile


commonlogger = logging.getLogger('fingerpointing common')


class LogInfo(object):

    def __init__(self, config, logformat=LOG_FORMAT):
        self.logger = logging.getLogger(PROJECTNAME)
        self.logfile = None
        self.handler = None
        self.configure(config)

    @property
    def has_log_file(self):
        return bool(self.logfile)

    def configure(self, config, logformat=LOG_FORMAT):
        self.logfile = config.get('audit-log', None)
        if self.logfile is None:
            commonlogger.warn(
                'No audit log file specified; audit log view will be disabled')
            return None
        self.logger.setLevel(logging.INFO)
        # first remove old handler if set:
        if self.handler is not None:
            self.logger.removeHandler(self.handler)
        # if either of maxBytes or backupCount is zero, rollover never occurs
        maxBytes = int(config.get('audit-log-max-size', 0))
        backupCount = int(config.get('audit-log-old-files', 0))
        self.handler = logging.handlers.RotatingFileHandler(
            self.logfile,
            maxBytes=maxBytes,
            backupCount=backupCount,
            delay=True,  # defer file creation to first emit
        )
        formatter = logging.Formatter(logformat)
        self.handler.setFormatter(formatter)
        self.logger.addHandler(self.handler)
        commonlogger.info('Logging audit information to ' + self.logfile)

    def __call__(self, *args, **kwargs):
        """Log information to a file handling access from multiple instances.
        This code was taken from ZEO/ClientStorage.py.
        """
        # if no logfile was specified just log the event normally
        if self.logfile is None:
            self.logger.info(*args, **kwargs)
            return

        # otherwise, try to lock the self.logfile then writing to it
        lockfilename = self.logfile + '.lock'
        n = 0

        while 1:
            try:
                # A content_template with hostname makes the implementation
                # container save, where each process in the different
                # containers is likely to have the same PID.
                # https://pypi.python.org/pypi/zc.lockfile#hostname-in-lock-file
                lock = zc.lockfile.LockFile(
                    lockfilename,
                    content_template='{pid};{hostname}',
                )
                try:
                    self.logger.info(*args, **kwargs)
                finally:
                    # even if logging fails: release the lock.
                    lock.close()
            except zc.lockfile.LockError:
                time.sleep(0.01)
                n += 1
                if n > 60000:
                    raise
            else:
                break


log_info = LogInfo(fingerpointing_config)



# -*- coding: utf-8 -*-
"""Tests for iterate subscribers.

Dexterity content types have different names.
"""
from collective.fingerpointing.interfaces import IFingerPointingSettings
from collective.fingerpointing.testing import INTEGRATION_TESTING
from collective.fingerpointing.testing import IS_PLONE_5
from collective.fingerpointing.testing import QIBBB
from logging import INFO
from plone import api
from plone.app.iterate.interfaces import ICheckinCheckoutPolicy
from testfixtures import LogCapture

import unittest


class IterateSubscribersTestCase(unittest.TestCase, QIBBB):
    """Tests for plone.app.iterate subscribers."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self._disable_lifecycle_events()

        with api.env.adopt_roles(['Manager']):
            self.folder = api.content.create(self.portal, 'Folder', 'folder')
        self.doc = api.content.create(self.folder, 'Document', 'doc')

    def _disable_lifecycle_events(self):
        name = IFingerPointingSettings.__identifier__ + '.audit_lifecycle'
        api.portal.set_registry_record(name, value=False)

    def test_checkout_checkin(self):
        if IS_PLONE_5:
            expected = (
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=checkout object=<Document at doc> working_copy=<Document at copy_of_doc>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=checkin object=<Document at copy_of_doc> baseline=<Document at doc>'),  # noqa: E501
            )
        else:
            expected = (
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=checkout object=<ATDocument at doc> working_copy=<ATDocument at copy_of_doc>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=checkin object=<ATDocument at copy_of_doc> baseline=<ATDocument at doc>'),  # noqa: E501
            )

        with LogCapture('collective.fingerpointing', level=INFO) as log:
            wc = ICheckinCheckoutPolicy(self.doc).checkout(self.folder)
            self.doc = ICheckinCheckoutPolicy(wc).checkin('updated')
            log.check(*expected)

    def test_checkout_cancel(self):
        if IS_PLONE_5:
            expected = (
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=checkout object=<Document at doc> working_copy=<Document at copy_of_doc>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=cancel checkout object=<Document at copy_of_doc> baseline=<Document at doc>'),  # noqa: E501
            )
        else:
            expected = (
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=checkout object=<ATDocument at doc> working_copy=<ATDocument at copy_of_doc>'),  # noqa: E501
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=cancel checkout object=<ATDocument at copy_of_doc> baseline=<ATDocument at doc>'),  # noqa: E501
            )

        with LogCapture('collective.fingerpointing', level=INFO) as log:
            wc = ICheckinCheckoutPolicy(self.doc).checkout(self.folder)
            ICheckinCheckoutPolicy(wc).cancelCheckout()
            log.check(*expected)

    def test_susbcriber_ignored_when_package_not_installed(self):
        # iterate events should not raise errors if package not installed
        self.uninstall()  # BBB: QI compatibility

        wc = ICheckinCheckoutPolicy(self.doc).checkout(self.folder)
        ICheckinCheckoutPolicy(wc).cancelCheckout()



# -*- coding: utf-8 -*-
from collective.fingerpointing.testing import INTEGRATION_TESTING
from plone import api

import unittest


class BaseUpgradeTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING
    profile_id = u'collective.fingerpointing:default'

    def setUp(self):
        self.portal = self.layer['portal']
        self.setup = self.portal['portal_setup']
        self.setup.setLastVersionForProfile(self.profile_id, self.from_)

    def _get_upgrade_step_by_title(self, title):
        """Return the upgrade step that matches the title specified."""
        self.setup.setLastVersionForProfile(self.profile_id, self.from_)
        upgrades = self.setup.listUpgrades(self.profile_id)
        steps = [s for s in upgrades[0] if s['title'] == title]
        return steps[0] if steps else None

    def _do_upgrade(self, step):
        """Execute an upgrade step."""
        request = self.layer['request']
        request.form['profile_id'] = self.profile_id
        request.form['upgrades'] = [step['id']]
        self.setup.manage_doUpgrades(request=request)


class To2TestCase(BaseUpgradeTestCase):

    from_ = '1'
    to_ = '2'

    def test_profile_version(self):
        version = self.setup.getLastVersionForProfile(self.profile_id)[0]
        self.assertEqual(version, self.from_)

    def test_registered_steps(self):
        steps = len(self.setup.listUpgrades(self.profile_id)[0])
        self.assertEqual(steps, 2)

    def test_update_configlet(self):
        # check if the upgrade step is registered
        title = u'Update control panel configlet'
        step = self._get_upgrade_step_by_title(title)
        self.assertIsNotNone(step)

        # simulate state on previous version
        cptool = api.portal.get_tool('portal_controlpanel')
        configlet = cptool.getActionObject('Products/fingerpointing')
        configlet.permissions = old_permissions = ('cmf.ManagePortal',)
        self.assertEqual(configlet.getPermissions(), old_permissions)

        # run the upgrade step to validate the update
        self._do_upgrade(step)
        configlet = cptool.getActionObject('Products/fingerpointing')
        new_permissions = ('collective.fingerpointing: Setup',)
        self.assertEqual(configlet.getPermissions(), new_permissions)

    def test_update_user_actions(self):
        # check if the upgrade step is registered
        title = u'Update user actions'
        step = self._get_upgrade_step_by_title(title)
        self.assertIsNotNone(step)

        # simulate state on previous version
        permission = 'collective.fingerpointing: View Audit Log'
        self.portal.manage_permission(
            permission_to_manage=permission, roles=[])
        roles = self.portal.rolesOfPermission(permission)
        roles = [r['name'] for r in roles if r['selected']]
        self.assertListEqual(roles, [])

        user_actions = self.portal['portal_actions'].user
        del user_actions['audit-log']
        self.assertNotIn('audit-log', user_actions)

        # run the upgrade step to validate the update
        self._do_upgrade(step)
        roles = self.portal.rolesOfPermission(permission)
        roles = [r['name'] for r in roles if r['selected']]  # noqa: E501; pylint: disable=W1662
        self.assertListEqual(roles, ['Manager', 'Site Administrator'])

        permissions = user_actions['audit-log'].permissions
        expected = (permission,)
        self.assertEqual(permissions, expected)
        url_expr = user_actions['audit-log'].url_expr
        self.assertIn('/@@fingerpointing-audit-log', url_expr)


class To3TestCase(BaseUpgradeTestCase):

    from_ = '2'
    to_ = '3'

    def test_profile_version(self):
        version = self.setup.getLastVersionForProfile(self.profile_id)[0]
        self.assertEqual(version, self.from_)

    def test_registered_steps(self):
        steps = len(self.setup.listUpgrades(self.profile_id)[0])
        self.assertEqual(steps, 1)

    def test_update_user_actions(self):
        # check if the upgrade step is registered
        title = u'Update user actions'
        step = self._get_upgrade_step_by_title(title)
        self.assertIsNotNone(step)

        # simulate state on previous version
        user_actions = self.portal['portal_actions'].user
        user_actions.moveObjectsDown('audit-log')
        self.assertEqual(user_actions.keys()[-1], 'audit-log')
        self.assertEqual(user_actions.keys()[-2], 'logout')

        # run the upgrade step to validate the update
        self._do_upgrade(step)
        self.assertEqual(user_actions.keys()[-1], 'logout')
        self.assertEqual(user_actions.keys()[-2], 'audit-log')


class To4TestCase(BaseUpgradeTestCase):

    from_ = '3'
    to_ = '4'

    def test_profile_version(self):
        version = self.setup.getLastVersionForProfile(self.profile_id)[0]
        self.assertEqual(version, self.from_)

    def test_registered_steps(self):
        steps = len(self.setup.listUpgrades(self.profile_id)[0])
        self.assertEqual(steps, 1)

    def test_update_user_actions(self):
        # check if the upgrade step is registered
        title = u'Add new field audit_workflow to configlet'
        step = self._get_upgrade_step_by_title(title)
        self.assertIsNotNone(step)

        from collective.fingerpointing.interfaces import IFingerPointingSettings  # noqa: E501
        from plone.registry.interfaces import IRegistry
        from zope.component import getUtility
        registry = getUtility(IRegistry)

        # simulate state on previous version
        record = IFingerPointingSettings.__identifier__ + '.audit_workflow'
        del registry.records[record]

        with self.assertRaises(KeyError):
            registry.forInterface(IFingerPointingSettings)

        # execute upgrade step and verify changes were applied
        self._do_upgrade(step)
        settings = registry.forInterface(IFingerPointingSettings)
        self.assertTrue(hasattr(settings, 'audit_workflow'))
        self.assertEqual(settings.audit_workflow, True)


class To5TestCase(BaseUpgradeTestCase):

    from_ = '4'
    to_ = '5'

    def test_profile_version(self):
        version = self.setup.getLastVersionForProfile(self.profile_id)[0]
        self.assertEqual(version, self.from_)

    def test_registered_steps(self):
        steps = len(self.setup.listUpgrades(self.profile_id)[0])
        self.assertEqual(steps, 1)

    def test_update_user_actions(self):
        # check if the upgrade step is registered
        title = u'Add new field audit_profile_imports to configlet'
        step = self._get_upgrade_step_by_title(title)
        self.assertIsNotNone(step)

        from collective.fingerpointing.interfaces import IFingerPointingSettings  # noqa: E501
        from plone.registry.interfaces import IRegistry
        from zope.component import getUtility
        registry = getUtility(IRegistry)

        # simulate state on previous version
        record = IFingerPointingSettings.__identifier__ + '.audit_profile_imports'  # noqa: E501
        del registry.records[record]

        with self.assertRaises(KeyError):
            registry.forInterface(IFingerPointingSettings)

        # execute upgrade step and verify changes were applied
        self._do_upgrade(step)
        settings = registry.forInterface(IFingerPointingSettings)
        self.assertTrue(hasattr(settings, 'audit_profile_imports'))
        self.assertEqual(settings.audit_profile_imports, True)



# -*- coding: utf-8 -*-
from collective.fingerpointing.config import PROJECTNAME
from collective.fingerpointing.logger import commonlogger


def update_configlet(setup_tool):
    """Update control panel configlet."""
    profile = 'profile-{0}:default'.format(PROJECTNAME)
    setup_tool.runImportStepFromProfile(profile, 'controlpanel')
    commonlogger.info('Control panel configlet updated.')


def update_user_actions(setup_tool):
    """Update user actions."""
    profile = 'profile-{0}:default'.format(PROJECTNAME)
    setup_tool.runImportStepFromProfile(profile, 'rolemap')
    commonlogger.info('Permissions updated.')
    setup_tool.runImportStepFromProfile(profile, 'actions')
    commonlogger.info('User actions updated.')



# -*- coding: utf-8 -*-
from collective.fingerpointing.interfaces import IFingerPointingSettings
from collective.fingerpointing.testing import INTEGRATION_TESTING
from collective.fingerpointing.testing import QIBBB
from plone import api
from plone.app.testing import logout
from plone.registry.interfaces import IRegistry
from zope.component import getUtility

import unittest


class ControlPanelTestCase(unittest.TestCase, QIBBB):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.controlpanel = self.portal['portal_controlpanel']

    def test_controlpanel_has_view(self):
        view = api.content.get_view(
            u'fingerpointing-settings', self.portal, self.request)
        view = view.__of__(self.portal)
        self.assertTrue(view())

    def test_controlpanel_view_is_protected(self):
        from AccessControl import Unauthorized
        logout()
        with self.assertRaises(Unauthorized):
            self.portal.restrictedTraverse('@@fingerpointing-settings')

    def test_controlpanel_installed(self):
        actions = [
            a.getAction(self)['id'] for a in self.controlpanel.listActions()]
        self.assertIn('fingerpointing', actions)

    def test_controlpanel_removed_on_uninstall(self):
        self.uninstall()  # BBB: QI compatibility

        actions = [
            a.getAction(self)['id'] for a in self.controlpanel.listActions()]
        self.assertNotIn('fingerpointing', actions)


class RegistryTestCase(unittest.TestCase, QIBBB):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.registry = getUtility(IRegistry)
        self.settings = self.registry.forInterface(IFingerPointingSettings)

    def test_audit_pas_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'audit_pas'))
        self.assertEqual(self.settings.audit_pas, True)

    def test_audit_lifecycle_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'audit_lifecycle'))
        self.assertEqual(self.settings.audit_lifecycle, True)

    def test_audit_workflow_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'audit_workflow'))
        self.assertEqual(self.settings.audit_workflow, True)

    def test_audit_profile_imports_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'audit_profile_imports'))
        self.assertEqual(self.settings.audit_profile_imports, True)

    def test_audit_registry_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'audit_registry'))
        self.assertEqual(self.settings.audit_registry, True)

    def test_audit_iterate_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'audit_iterate'))
        self.assertEqual(self.settings.audit_iterate, True)

    def test_records_removed_on_uninstall(self):
        self.uninstall()  # BBB: QI compatibility

        records = [
            IFingerPointingSettings.__identifier__ + '.audit_pas',
            IFingerPointingSettings.__identifier__ + '.audit_lifecycle',
            IFingerPointingSettings.__identifier__ + '.audit_workflow',
            IFingerPointingSettings.__identifier__ + '.audit_profile_imports',
            IFingerPointingSettings.__identifier__ + '.audit_registry',
            IFingerPointingSettings.__identifier__ + '.audit_iterate',
        ]
        for r in records:
            self.assertNotIn(r, self.registry)



# -*- coding: utf-8 -*-
from collective.fingerpointing.testing import ROBOT_TESTING
from plone.testing import layered

import os
import robotsuite
import unittest


def test_suite():
    suite = unittest.TestSuite()
    current_dir = os.path.abspath(os.path.dirname(__file__))
    tests = [
        doc for doc in os.listdir(current_dir)
        if doc.startswith('test_') and doc.endswith('.robot')
    ]
    suite.addTests([
        layered(robotsuite.RobotTestSuite(t), layer=ROBOT_TESTING)
        for t in tests
    ])
    return suite



# -*- coding: utf-8 -*-
from collective.fingerpointing.config import AUDIT_MESSAGE
from collective.fingerpointing.interfaces import IFingerPointingSettings
from collective.fingerpointing.logger import log_info
from collective.fingerpointing.utils import get_request_information
from plone import api
from zope.component import ComponentLookupError
from zope.lifecycleevent import IObjectAddedEvent
from zope.lifecycleevent import IObjectModifiedEvent
from zope.lifecycleevent import IObjectRemovedEvent


def lifecycle_logger(obj, event):
    """Log content type life cycle events like object creation,
    modification and removal.
    """
    name = IFingerPointingSettings.__identifier__ + '.audit_lifecycle'
    try:
        audit_lifecycle = api.portal.get_registry_record(name, default=False)
    except ComponentLookupError:  # Plone site removed
        return

    if not audit_lifecycle:
        return

    user, ip = get_request_information()

    if IObjectAddedEvent.providedBy(event):
        action = 'create'
    elif IObjectModifiedEvent.providedBy(event):
        action = 'modify'
    elif IObjectRemovedEvent.providedBy(event):
        action = 'remove'
    else:  # should never happen
        action = '-'

    extras = 'object=' + repr(obj)
    log_info(AUDIT_MESSAGE.format(user, ip, action, extras))



# -*- coding: utf-8 -*-
"""Setup testing infrastructure.

For Plone 5 we need to install plone.app.contenttypes.
"""
from collective.fingerpointing.config import PROJECTNAME
from plone import api
from plone.app.robotframework.testing import AUTOLOGIN_LIBRARY_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2

import os
import pkg_resources
import shutil
import tempfile


try:
    pkg_resources.get_distribution('plone.app.contenttypes')
except pkg_resources.DistributionNotFound:
    from plone.app.testing import PLONE_FIXTURE
else:
    from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE as PLONE_FIXTURE  # noqa: E501

try:
    pkg_resources.get_distribution('plone.app.iterate')
except pkg_resources.DistributionNotFound:
    HAS_ITERATE = False
else:
    HAS_ITERATE = True

IS_PLONE_5 = api.env.plone_version().startswith('5')
IS_BBB = api.env.plone_version().startswith('4.3')


class QIBBB:
    """BBB: remove on deprecation of Plone 4.3."""
    def uninstall(self):
        if IS_BBB:
            qi = self.portal['portal_quickinstaller']
            with api.env.adopt_roles(['Manager']):
                qi.uninstallProducts([PROJECTNAME])
        else:
            from Products.CMFPlone.utils import get_installer
            qi = get_installer(self.portal, self.request)
            with api.env.adopt_roles(['Manager']):
                qi.uninstall_product(PROJECTNAME)
        return qi


class Fixture(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    # XXX: this can probably be done in a different way
    def _setup_audit_log(self):
        """Fake configuration for collective.fingerpointing as tests
        expect an audit.log file to be set.
        """
        from collective.fingerpointing.config import fingerpointing_config
        from collective.fingerpointing.logger import log_info
        self.temp_dir = tempfile.mkdtemp()
        fingerpointing_config['audit-log'] = os.path.join(
            self.temp_dir,
            'audit.log',
        )
        log_info.configure(fingerpointing_config)

    def _cleanup_audit_log(self):
        from collective.fingerpointing.config import fingerpointing_config
        fingerpointing_config['audit-log'] = None
        shutil.rmtree(self.temp_dir)

    def setUpZope(self, app, configurationContext):
        self._setup_audit_log()
        if HAS_ITERATE:
            import plone.app.iterate
            self.loadZCML(package=plone.app.iterate)

        import collective.fingerpointing
        self.loadZCML(package=collective.fingerpointing)

    def setUpPloneSite(self, portal):
        if HAS_ITERATE:
            self.applyProfile(portal, 'plone.app.iterate:plone.app.iterate')

        self.applyProfile(portal, 'collective.fingerpointing:default')
        portal.portal_workflow.setDefaultChain('simple_publication_workflow')

    def tearDown(self):
        super(Fixture, self).tearDown()
        self._cleanup_audit_log()


FIXTURE = Fixture()

INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,), name='collective.fingerpointing:Integration')

FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE,), name='collective.fingerpointing:Functional')

ROBOT_TESTING = FunctionalTesting(
    bases=(FIXTURE, AUTOLOGIN_LIBRARY_FIXTURE, z2.ZSERVER_FIXTURE),
    name='collective.fingerpointing:Robot',
)



# -*- coding: utf-8 -*-
"""Event subscriber handler for IActionSucceededEvent."""
from collective.fingerpointing.config import AUDIT_MESSAGE
from collective.fingerpointing.interfaces import IFingerPointingSettings
from collective.fingerpointing.logger import log_info
from collective.fingerpointing.utils import get_request_information
from plone import api


def workflow_logger(event):
    """Log workflow transitions."""
    name = IFingerPointingSettings.__identifier__ + '.audit_workflow'
    audit_workflow = api.portal.get_registry_record(name, default=False)
    if not audit_workflow:
        return

    user, ip = get_request_information()

    action = 'workflow transition'
    extras = 'object={0} transition={1}'.format(
        event.object,
        event.action,
    )
    log_info(AUDIT_MESSAGE.format(user, ip, action, extras))



# -*- coding: utf-8 -*-
from App.config import getConfiguration


PROJECTNAME = 'collective.fingerpointing'
AUDIT_MESSAGE = 'user={0} ip={1} action={2} {3}'
LOG_FORMAT = '%(asctime)s - %(levelname)s - %(message)s'

# read configuration from zope.conf
zopeConf = getConfiguration()
product_config = getattr(zopeConf, 'product_config', {})
fingerpointing_config = product_config.get(PROJECTNAME, {})



# -*- coding: utf-8 -*-
from collective.fingerpointing.logger import log_info
from collective.fingerpointing.utils import all_audit_log_lines
from Products.Five.browser import BrowserView


DEFAULT_SIZE = 100


class LogView(BrowserView):

    @property
    def available(self):
        return log_info.has_log_file

    @property
    def page(self):
        try:
            return int(self.request.form.get('page', 1))
        except ValueError:
            return 1

    @property
    def size(self):
        try:
            return int(self.request.form.get('size', DEFAULT_SIZE))
        except ValueError:
            return DEFAULT_SIZE

    @property
    def query(self):
        return self.request.form.get('query', '').strip()

    def _filtered_audit_log_lines(self):
        """Generator for requested filtered log lines.
        """
        query = self.query
        for line in all_audit_log_lines():
            if not query or query in line:
                yield line

    def _batched_audit_log_lines(self):
        """Generator for requested batch of the already filtered log lines.
        """
        start = (self.page - 1) * self.size
        end = start + self.size
        for pos, line in enumerate(self._filtered_audit_log_lines()):
            if pos >= end:
                break
            if pos < start:
                continue
            yield line

    def build_audit_log(self):
        """Triggers build and sets attributes for use in template.
        """
        lines = [l for l in self._batched_audit_log_lines()]
        self.audit_log = '\n'.join(lines)
        self.has_next = len(lines) == self.size
        self.has_prev = self.page > 1

    @property
    def base_url(self):
        return self.context.absolute_url() + '/@@fingerpointing-audit-log'

    def pagination_url(self, type_, size=None):
        params = {'size': size if size is not None else self.size}
        if self.query:
            params['query'] = self.query
        if type_ == 'first':
            params['page'] = 1
        if type_ == 'prev':
            params['page'] = self.page - 1
        if type_ == 'next':
            params['page'] = self.page + 1
        return self.base_url + '?' + '&'.join(
            [k + '=' + str(v) for k, v in params.items()])



# -*- coding: utf-8 -*-
from collective.fingerpointing.logger import log_info
from file_read_backwards import FileReadBackwards
from glob import glob
from plone import api
from zope.globalrequest import getRequest

import os


def get_request_information():
    """Return logged in user name and remote IP address."""
    request = getRequest()
    try:
        user_id = api.user.get_current().getMemberId()
    except AttributeError:
        user_id = '-'  # anonymous user

    # honor Cloudflare real client IP address request header if present
    # see: https://support.cloudflare.com/hc/en-us/articles/200170986
    if 'HTTP_CF_CONNECTING_IP' in request.environ:
        ip = request.environ['HTTP_CF_CONNECTING_IP']
    # Common proxy configuration
    elif 'HTTP_X_FORWARDED_FOR' in request.environ:
        ip = request.environ['HTTP_X_FORWARDED_FOR']
    else:
        ip = request.getClientAddr() or 'None'  # return 'None' on tests

    return user_id, ip


def audit_log_files():
    """Return a list with the current audit log and all of its
    backups. The list is sorted with newer files first.
    """
    logfiles = []
    if log_info.has_log_file:
        # retrieve the list of audit log backup files
        logfiles = sorted(
            [f for f in glob(log_info.logfile + '.*') if '.lock' not in f],
            reverse=True,
        )
        # include active audit log as first element
        if os.path.exists(log_info.logfile):
            logfiles.insert(0, log_info.logfile)
    return logfiles


def audit_log_lines_of(logfile):
    """Generator for log lines of a log file reversed.
    """
    with FileReadBackwards(logfile, encoding='utf-8') as frb:
        for line in frb:
            yield line.strip()


def all_audit_log_lines():
    """Generator for all lines in the audit logs.

    - Newest file first
    - File is read reversed (backwards)
    """
    for logfile in audit_log_files():
        for line in audit_log_lines_of(logfile):
            yield line



# -*- coding: utf-8 -*-
from collective.fingerpointing import _
from collective.fingerpointing.interfaces import IFingerPointingSettings
from plone.app.registry.browser import controlpanel


class FingerPointingSettingsEditForm(controlpanel.RegistryEditForm):

    """Control panel edit form."""

    schema = IFingerPointingSettings
    label = _(u'Finger Pointing')
    description = _(u'Settings for the collective.fingerpointing package')


class FingerPointingSettingsControlPanel(controlpanel.ControlPanelFormWrapper):

    """Control panel form wrapper."""

    form = FingerPointingSettingsEditForm



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup


version = '1.8'
description = 'Keep track of different events and write them down to an audit log.'
long_description = (
    open('README.rst').read() + '\n' +
    open('CONTRIBUTORS.rst').read() + '\n' +
    open('CHANGES.rst').read()
)

setup(
    name='collective.fingerpointing',
    version=version,
    description=description,
    long_description=long_description,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Plone :: 4.3',
        'Framework :: Plone :: 5.1',
        'Framework :: Plone',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Logging',
    ],
    keywords='plone events subscribers log audit security',
    author='Hector Velarde',
    author_email='hector.velarde@gmail.com',
    url='https://github.com/collective/collective.fingerpointing',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['collective'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'file_read_backwards',
        'plone.api >=1.5.1',
        'plone.app.registry',
        'plone.registry',
        'Products.CMFCore',
        'Products.CMFPlone >=4.3',
        'Products.GenericSetup',
        'Products.PlonePAS >=5.0.9',
        'Products.PluggableAuthService >=1.11.0',
        'setuptools',
        'six',
        'zc.lockfile>=1.2.1',
        'zope.component',
        'zope.globalrequest',
        'zope.i18nmessageid',
        'zope.interface',
        'zope.lifecycleevent',
        'zope.schema',
    ],
    extras_require={
        'test': [
            'AccessControl',
            'plone.app.dexterity',
            'plone.app.iterate',
            'plone.app.robotframework',
            'plone.app.testing [robot]',
            'plone.browserlayer',
            'plone.testing',
            'robotsuite',
            'testfixtures',
            'zope.component',
            'zope.event',
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



# -*- coding: utf-8 -*-
from collective.fingerpointing import _
from zope import schema
from zope.interface import Interface


class IBrowserLayer(Interface):

    """A layer specific for this add-on product."""


class IFingerPointingSettings(Interface):

    """Schema for the control panel form."""

    audit_pas = schema.Bool(
        title=_(u'Audit Authentication Service?'),
        description=_(
            u'Log authentication events like users logging in and loggin out.'),   # noqa: E501
        default=True,
    )

    audit_lifecycle = schema.Bool(
        title=_(u'Audit Content Type Life Cycle?'),
        description=_(
            u'Log content type life cycle events like object creation, modification and removal.'),  # noqa: E501
        default=True,
    )

    audit_workflow = schema.Bool(
        title=_(u'Audit Workflow Transitions?'),
        description=_(u'Log workflow transitions like object publishing.'),
        default=True,
    )

    audit_profile_imports = schema.Bool(
        title=_(u'Audit Profile Imports?'),
        description=_(
            u'Log Generic Setup profile imports; '
            u'this is useful to audit add-on installs/uninstalls.',
        ),
        default=True,
    )

    audit_registry = schema.Bool(
        title=_(u'Audit Registry?'),
        description=_(u'Log registry events like records being modified.'),
        default=True,
    )

    audit_iterate = schema.Bool(
        title=_(u'Audit Iterate?'),
        description=_(
            u'If plone.app.iterate is installed, '
            u'log events like content editing checkouts and checkins.'),
        default=True,
    )



# -*- coding: utf-8 -*-
"""Tests for lifecycle subscriber."""
from collective.fingerpointing.testing import INTEGRATION_TESTING
from collective.fingerpointing.testing import QIBBB
from logging import INFO
from Products.PlonePAS.events import UserLoggedInEvent
from Products.PlonePAS.events import UserLoggedOutEvent
from Products.PluggableAuthService.events import GroupDeleted
from Products.PluggableAuthService.events import PrincipalCreated
from Products.PluggableAuthService.events import PrincipalDeleted
from testfixtures import LogCapture
from zope.event import notify

import unittest


class PasSubscribersTestCase(unittest.TestCase, QIBBB):
    """Tests for PluggableAuthService subscribers."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

    def test_user_login(self):
        event = UserLoggedInEvent(self.request)
        with LogCapture('collective.fingerpointing', level=INFO) as log:
            notify(event)
            log.check(
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=login '),  # noqa: E501
            )

    def test_user_logout(self):
        event = UserLoggedOutEvent(self.request)
        with LogCapture('collective.fingerpointing', level=INFO) as log:
            notify(event)
            log.check(
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=logout '),  # noqa: E501
            )

    def test_user_created(self):
        event = PrincipalCreated('foo')
        with LogCapture('collective.fingerpointing', level=INFO) as log:
            notify(event)
            log.check(
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=create principal=foo'),  # noqa: E501
            )

    def test_user_removed(self):
        event = PrincipalDeleted('foo')
        with LogCapture('collective.fingerpointing', level=INFO) as log:
            notify(event)
            log.check(
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=remove user=foo'),  # noqa: E501
            )

    def test_group_removed(self):
        event = GroupDeleted('bar')
        with LogCapture('collective.fingerpointing', level=INFO) as log:
            notify(event)
            log.check(
                ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=remove group=bar'),  # noqa: E501
            )

    def test_susbcriber_ignored_when_package_not_installed(self):
        # authentication events should not raise errors
        # if package is not installed
        self.uninstall()  # BBB: QI compatibility

        event = UserLoggedInEvent(self.request)
        notify(event)
        event = UserLoggedOutEvent(self.request)
        notify(event)
        event = PrincipalCreated('foo')
        notify(event)
        event = PrincipalDeleted('foo')
        notify(event)



# -*- coding: utf-8 -*-
from collective.fingerpointing.testing import INTEGRATION_TESTING
from plone import api
from plone.app.testing import logout

import unittest


class LogViewTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.view = api.content.get_view(
            'fingerpointing-audit-log', self.portal, self.request)

    def test_audit_log_view(self):
        view = self.view.__of__(self.portal)
        self.assertTrue(view())

    def test_audit_log_view_is_protected(self):
        from AccessControl import Unauthorized
        logout()
        with self.assertRaises(Unauthorized):
            self.portal.restrictedTraverse('@@fingerpointing-audit-log')

    def test_available(self):
        self.assertTrue(self.view.available)



# -*- coding: utf-8 -*-
from collective.fingerpointing.config import AUDIT_MESSAGE
from collective.fingerpointing.interfaces import IFingerPointingSettings
from collective.fingerpointing.logger import log_info
from collective.fingerpointing.utils import get_request_information
from plone import api
from plone.app.iterate.interfaces import ICancelCheckoutEvent
from plone.app.iterate.interfaces import ICheckinEvent
from plone.app.iterate.interfaces import ICheckoutEvent


def iterate_logger(event):
    """Log events like content editing checkouts and checkins, if
    plone.app.iterate is installed.
    """
    name = IFingerPointingSettings.__identifier__ + '.audit_iterate'
    audit_iterate = api.portal.get_registry_record(name, default=False)
    if not audit_iterate:
        return

    user, ip = get_request_information()

    if ICheckoutEvent.providedBy(event):
        action = 'checkout'
        extras = 'object={0} working_copy={1}'.format(
            event.object, event.working_copy)
    elif ICheckinEvent.providedBy(event):
        action = 'checkin'
        extras = 'object={0} baseline={1}'.format(
            event.object, event.baseline)
    elif ICancelCheckoutEvent.providedBy(event):
        action = 'cancel checkout'
        extras = 'object={0} baseline={1}'.format(
            event.object, event.baseline)
    else:  # should never happen
        action = '-'
        extras = 'object=' + repr(event.object)

    log_info(AUDIT_MESSAGE.format(user, ip, action, extras))



# -*- coding: utf-8 -*-
"""Tests for Generic Setup subscriber."""
from collective.fingerpointing.config import PROJECTNAME
from collective.fingerpointing.interfaces import IFingerPointingSettings
from collective.fingerpointing.testing import INTEGRATION_TESTING
from collective.fingerpointing.testing import IS_BBB
from logging import INFO
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from testfixtures import LogCapture

import unittest


ADD_ON = 'plone.session'


class GenericSetupSubscribersTestCase(unittest.TestCase):
    """Tests Generic Setup subscribers."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        # disable registry audit
        name = IFingerPointingSettings.__identifier__ + '.audit_registry'
        api.portal.set_registry_record(name, False)

    @property
    def qi(self):
        if IS_BBB:  # Plone 4.3
            return self.portal['portal_quickinstaller']
        else:
            from Products.CMFPlone.utils import get_installer
            return get_installer(self.portal, self.request)

    @unittest.skipIf(IS_BBB, 'Plone >= 5.1')
    def test_profile_imports(self):
        expected = (
            ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=profile imported id=plone.session:default version=1001'),  # noqa: E501
            ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=profile imported id=plone.session:uninstall version=-'),  # noqa: E501
        )
        with LogCapture('collective.fingerpointing', level=INFO) as log:
            self.qi.install_product(ADD_ON)
            self.qi.uninstall_product(ADD_ON)
            log.check(*expected)

    @unittest.skipUnless(IS_BBB, 'Plone 4.3')
    def test_profile_imports_BBB(self):
        expected = (
            ('collective.fingerpointing', 'INFO', 'user=test_user_1_ ip=None action=profile imported id=plone.session:default version=1000'),  # noqa: E501
        )

        with LogCapture('collective.fingerpointing', level=INFO) as log:
            self.qi.installProduct(ADD_ON)
            self.qi.uninstallProducts([ADD_ON])
            log.check(*expected)

    @unittest.skipIf(IS_BBB, 'Plone >= 5.1')
    def test_susbcriber_ignored_when_package_not_installed(self):
        # events should not raise errors if package is not installed
        self.qi.uninstall_product(PROJECTNAME)
        self.qi.install_product(ADD_ON)
        self.qi.uninstall_product(ADD_ON)

    @unittest.skipUnless(IS_BBB, 'Plone 4.3')
    def test_susbcriber_ignored_when_package_not_installed_BBB(self):
        # events should not raise errors if package is not installed
        self.qi.uninstallProducts([PROJECTNAME])
        self.qi.installProduct(ADD_ON)
        self.qi.uninstallProducts([ADD_ON])



# -*- coding: utf-8 -*-
from .genericsetup_logger import profile_imports_logger  # noqa: F401
from .lifecycle_logger import lifecycle_logger  # noqa: F401
from .pas_logger import pas_logger  # noqa: F401
from .registry_logger import registry_logger  # noqa: F401
from .workflow_logger import workflow_logger  # noqa: F401

import pkg_resources


try:
    # plone.app.iterate is dependency of the "Plone" package, but not of
    # the Products.CMFPlone. So dependend on the way Plone is installed it
    # might be not available.
    pkg_resources.get_distribution('plone.app.iterate')
except pkg_resources.DistributionNotFound:
    pass
else:
    from .iterate_logger import iterate_logger  # noqa: F401



# -*- coding: utf-8 -*-
from collective.fingerpointing.config import AUDIT_MESSAGE
from collective.fingerpointing.interfaces import IFingerPointingSettings
from collective.fingerpointing.logger import log_info
from collective.fingerpointing.utils import get_request_information
from plone import api
from Products.PluggableAuthService.interfaces.events import IGroupDeletedEvent
from Products.PluggableAuthService.interfaces.events import IPrincipalCreatedEvent  # noqa: E501
from Products.PluggableAuthService.interfaces.events import IPrincipalDeletedEvent  # noqa: E501
from Products.PluggableAuthService.interfaces.events import IUserLoggedInEvent
from Products.PluggableAuthService.interfaces.events import IUserLoggedOutEvent
from zope.component.interfaces import ComponentLookupError


def pas_logger(event):
    """Log authentication events like users logging in and loggin out."""
    name = IFingerPointingSettings.__identifier__ + '.audit_pas'
    try:
        audit_pas = api.portal.get_registry_record(name, default=False)
    except ComponentLookupError:  # plonectl adduser
        return

    if not audit_pas:
        return

    user, ip = get_request_information()

    if IUserLoggedInEvent.providedBy(event):
        action = 'login'
        extras = ''
    elif IUserLoggedOutEvent.providedBy(event):
        action = 'logout'
        extras = ''
    elif IPrincipalCreatedEvent.providedBy(event):
        action = 'create'
        extras = 'principal=' + str(event.principal)
    elif IPrincipalDeletedEvent.providedBy(event):
        action = 'remove'
        extras = 'user=' + str(event.principal)
    elif IGroupDeletedEvent.providedBy(event):
        action = 'remove'
        extras = 'group=' + str(event.principal)
    else:  # should never happen
        action = 'UNKNOWN'
        extras = ''

    log_info(AUDIT_MESSAGE.format(user, ip, action, extras))



# -*- coding: utf-8 -*-
from collective.fingerpointing.config import AUDIT_MESSAGE
from collective.fingerpointing.interfaces import IFingerPointingSettings
from collective.fingerpointing.logger import log_info
from collective.fingerpointing.utils import get_request_information
from plone import api
from plone.registry.interfaces import IRecordModifiedEvent

import six


def safe_utf8(s):
    if isinstance(s, six.text_type):
        s = s.encode('utf-8')
    return s


def registry_logger(event):
    """Log registry events like records being modified."""
    name = IFingerPointingSettings.__identifier__ + '.audit_registry'
    audit_registry = api.portal.get_registry_record(name, default=False)
    if not audit_registry:
        return

    user, ip = get_request_information()

    if IRecordModifiedEvent.providedBy(event):
        action = 'modify'
        extras = 'object={0} value={1}'.format(
            event.record,
            safe_utf8(event.record.value),
        )
    else:  # should never happen
        action = '-'
        extras = 'object' + event.record

    log_info(AUDIT_MESSAGE.format(user, ip, action, extras))



# -*- coding: utf-8 -*-
"""Ensure add-on is properly installed and uninstalled."""
from collective.fingerpointing.config import PROJECTNAME
from collective.fingerpointing.interfaces import IBrowserLayer
from collective.fingerpointing.testing import INTEGRATION_TESTING
from collective.fingerpointing.testing import IS_BBB
from collective.fingerpointing.testing import QIBBB
from plone.browserlayer.utils import registered_layers

import unittest


class InstallTestCase(unittest.TestCase):
    """Ensure product is properly installed."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

    @unittest.skipIf(IS_BBB, 'Plone >= 5.1')
    def test_installed(self):
        from Products.CMFPlone.utils import get_installer
        qi = get_installer(self.portal, self.request)
        self.assertTrue(qi.is_product_installed(PROJECTNAME))

    @unittest.skipUnless(IS_BBB, 'Plone < 5.1')
    def test_installed_BBB(self):
        qi = self.portal['portal_quickinstaller']
        self.assertTrue(qi.isProductInstalled(PROJECTNAME))

    def test_addon_layer(self):
        self.assertIn(IBrowserLayer, registered_layers())

    def test_user_action(self):
        user_actions = self.portal['portal_actions'].user
        self.assertIn('audit-log', user_actions)

        permissions = user_actions['audit-log'].permissions
        expected = ('collective.fingerpointing: View Audit Log',)
        self.assertEqual(permissions, expected)
        url_expr = user_actions['audit-log'].url_expr
        self.assertIn('/@@fingerpointing-audit-log', url_expr)

    def test_setup_permission(self):
        permission = 'collective.fingerpointing: Setup'
        roles = self.portal.rolesOfPermission(permission)
        roles = [r['name'] for r in roles if r['selected']]
        expected = ['Manager', 'Site Administrator']
        self.assertListEqual(roles, expected)

    def test_show_log_permission(self):
        permission = 'collective.fingerpointing: View Audit Log'
        roles = self.portal.rolesOfPermission(permission)
        roles = [r['name'] for r in roles if r['selected']]
        expected = ['Manager', 'Site Administrator']
        self.assertListEqual(roles, expected)


class UninstallTestCase(unittest.TestCase, QIBBB):
    """Ensure product is properly uninstalled."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.qi = self.uninstall()  # BBB: QI compatibility

    @unittest.skipIf(IS_BBB, 'Plone >= 5.1')
    def test_uninstalled(self):
        self.assertFalse(self.qi.is_product_installed(PROJECTNAME))

    @unittest.skipUnless(IS_BBB, 'Plone < 5.1')
    def test_uninstalled_BBB(self):
        self.assertFalse(self.qi.isProductInstalled(PROJECTNAME))

    def test_addon_layer_removed(self):
        self.assertNotIn(IBrowserLayer, registered_layers())

    def test_user_action_removed(self):
        user_actions = self.portal['portal_actions'].user
        self.assertNotIn('audit-log', user_actions)
